import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

class ISearchDelegate extends SearchDelegate {
  ISearchDelegate({
    this.list,
    this.type = 'all_history',
  });

  final String type;
  final Function list;

  @override
  ThemeData appBarTheme(BuildContext context) {
    final ThemeData theme = Theme.of(context);
    return theme.copyWith(
      inputDecorationTheme: InputDecorationTheme(
        hintStyle: TextStyle(
          color: theme.primaryTextTheme.headline6.color.withOpacity(0.6),
        ),
      ),
      textTheme: theme.textTheme.copyWith(
        headline6: theme.textTheme.headline6.copyWith(
          color: theme.primaryTextTheme.headline6.color,
        ),
      ),
    );
  }

  @override
  List<Widget> buildActions(BuildContext context) {
    return [
      IconButton(
        icon: Icon(Icons.clear),
        onPressed: () {
          query = '';
        },
      ),
    ];
  }

  @override
  Widget buildLeading(BuildContext context) {
    return IconButton(
      icon: Icon(Icons.arrow_back),
      onPressed: () {
        close(context, null);
      },
    );
  }

  @override
  Widget buildResults(BuildContext context) {
    _saveList();
    return list(query);
  }

  @override
  Widget buildSuggestions(BuildContext context) {
    return SearchHistory(
      type: this.type,
      query: this.query,
      onClear: clearHistroy,
      onSelected: (String suggestions) {
        query = suggestions;
        showResults(context);
      },
    );
  }

  void clearHistroy() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    await prefs.remove(type);
  }

  void _saveList() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    List<String> history = [];
    var list = prefs.getStringList(type);
    if (list != null) history = list;
    if (query.isEmpty == false) history.insert(0, query);
    await prefs.setStringList(type, history);
  }
}

class SearchHistory extends StatefulWidget {
  final String type;
  final String query;
  final Function onClear;
  final ValueChanged<String> onSelected;

  SearchHistory({this.type, this.query, this.onClear, this.onSelected});

  @override
  _SearchHistoryState createState() => _SearchHistoryState();
}

class _SearchHistoryState extends State<SearchHistory> {
  List<String> history = [];

  @override
  void initState() {
    super.initState();
    _loadHistory();
  }

  _loadHistory() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    setState(() {
      history = prefs.getStringList(widget.type);
    });
  }

  @override
  Widget build(BuildContext context) {
    int count = history == null ? 0 : history.length;
    if (count == 0) return emptyHistory();
    return Column(
      children: <Widget>[
        header(),
        Expanded(
          child: ListView.builder(
            itemCount: count,
            physics: BouncingScrollPhysics(),
            itemBuilder: (BuildContext context, int i) {
              final String suggestion = history[i];
              return ListTile(
                title: Text(suggestion),
                onTap: () {
                  widget.onSelected(suggestion);
                },
                leading:
                    widget.query.isEmpty ? Icon(Icons.history) : Icon(null),
              );
            },
          ),
        )
      ],
    );
  }

  Widget emptyHistory() {
    return Container(
      child: Center(
        child: Text('Enter a few words to search'),
      ),
    );
  }

  Widget header() {
    if (history == null || history.length == 0) return Container();

    return ListTile(
      onTap: () {
        widget.onClear();
        widget.onSelected('');
      },
      leading: Icon(Icons.clear),
      title: Text('Clear search history'),
    );
  }
}
